from .core import (
    Matrix,
    generate_identity,
    generate_tridiagonal,
    generate_triband,
    generate_diagonal,
    generate_upper_triangular,
    generate_lower_triangular,
    generate_symmetric,
    generate_toeplitz,
    generate_hankel,
    generate_circulant,
    is_symmetric,
    is_toeplitz,
)

__all__ = [
    "Matrix",
    "generate_identity",
    "generate_tridiagonal",
    "generate_triband",
    "generate_diagonal",
    "generate_upper_triangular",
    "generate_lower_triangular",
    "generate_symmetric",
    "generate_toeplitz",
    "generate_hankel",
    "generate_circulant",
    "is_symmetric",
    "is_toeplitz",
]


import numpy as np

class Matrix:
    def __init__(self, data):
        self.data = np.array(data)

    def transpose(self):
        return Matrix(self.data.T)

    def add(self, other):
        if self.data.shape != other.data.shape:
            raise ValueError("Matrix dimensions must match for addition.")
        return Matrix(self.data + other.data)

    def multiply(self, other):
        if self.data.shape[1] != other.data.shape[0]:
            raise ValueError("Matrix A columns must match Matrix B rows for multiplication.")
        return Matrix(np.dot(self.data, other.data))

    def is_symmetric(self):
        return np.array_equal(self.data, self.data.T)

    def __repr__(self):
        return f"Matrix({self.data.tolist()})"


def generate_identity(n):
    return Matrix(np.eye(n, dtype=int))


def generate_tridiagonal(n, lower=-1, main=2, upper=-1):
    data = np.zeros((n, n), dtype=int)
    for i in range(n):
        data[i, i] = main
        if i > 0:
            data[i, i - 1] = lower
        if i < n - 1:
            data[i, i + 1] = upper
    return Matrix(data)


def generate_triband(n, bandwidth=1, value=1):
    data = np.zeros((n, n), dtype=int)
    for i in range(n):
        for j in range(max(0, i - bandwidth), min(n, i + bandwidth + 1)):
            data[i, j] = value
    return Matrix(data)


def generate_diagonal(values):
    return Matrix(np.diag(values))


def generate_upper_triangular(n, fill=1):
    data = np.zeros((n, n), dtype=int)
    for i in range(n):
        for j in range(i, n):
            data[i, j] = fill
    return Matrix(data)


def generate_lower_triangular(n, fill=1):
    data = np.zeros((n, n), dtype=int)
    for i in range(n):
        for j in range(i + 1):
            data[i, j] = fill
    return Matrix(data)


def generate_symmetric(base):
    base = np.array(base)
    return Matrix((base + base.T) // 2)


def generate_toeplitz(first_row, first_col):
    n = len(first_col)
    m = len(first_row)
    data = np.zeros((n, m), dtype=int)
    for i in range(n):
        for j in range(m):
            if i - j >= 0:
                data[i][j] = first_col[i - j]
            else:
                data[i][j] = first_row[j - i]
    return Matrix(data)


def generate_hankel(first_col, last_row):
    n = len(first_col)
    m = len(last_row)
    data = np.zeros((n, m), dtype=int)
    for i in range(n):
        for j in range(m):
            if i + j < len(first_col):
                data[i][j] = first_col[i + j]
            else:
                data[i][j] = last_row[i + j - len(first_col) + 1]
    return Matrix(data)


def generate_circulant(first_row):
    n = len(first_row)
    data = np.zeros((n, n), dtype=int)
    for i in range(n):
        data[i] = first_row[-i:] + first_row[:-i]
    return Matrix(data)


def is_symmetric(matrix):
    data = matrix.data if isinstance(matrix, Matrix) else np.array(matrix)
    return np.array_equal(data, data.T)


def is_toeplitz(matrix):
    data = matrix.data if isinstance(matrix, Matrix) else np.array(matrix)
    rows, cols = data.shape
    for i in range(1, rows):
        for j in range(1, cols):
            if data[i, j] != data[i - 1, j - 1]:
                return False
    return True


import pytest
from matrixx import (
    Matrix,
    generate_identity,
    generate_tridiagonal,
    generate_triband,
    generate_diagonal,
    generate_upper_triangular,
    generate_lower_triangular,
    generate_symmetric,
    generate_toeplitz,
    generate_hankel,
    generate_circulant,
    is_symmetric,
    is_toeplitz
)

def test_identity():
    m = generate_identity(3)
    assert (m.data == [[1,0,0],[0,1,0],[0,0,1]]).all()

def test_addition():
    a = Matrix([[1, 2], [3, 4]])
    b = Matrix([[5, 6], [7, 8]])
    c = a.add(b)
    assert (c.data == [[6, 8], [10, 12]]).all()

def test_multiplication():
    a = Matrix([[1, 2], [3, 4]])
    b = Matrix([[2, 0], [1, 2]])
    result = a.multiply(b)
    assert (result.data == [[4, 4], [10, 8]]).all()

def test_symmetric():
    m = generate_symmetric([[1, 2], [3, 4]])
    assert is_symmetric(m)

def test_toeplitz():
    row = [1, 2, 3]
    col = [1, 4, 5]
    t = generate_toeplitz(row, col)
    expected = [[1, 2, 3], [4, 1, 2], [5, 4, 1]]
    assert (t.data == expected).all()
    assert is_toeplitz(t)

def test_hankel():
    h = generate_hankel([1, 2, 3], [3, 4, 5])
    expected = [[1, 2, 3], [2, 3, 4], [3, 4, 5]]
    assert (h.data == expected).all()

def test_circulant():
    c = generate_circulant([1, 2, 3])
    expected = [[1, 2, 3], [3, 1, 2], [2, 3, 1]]
    assert (c.data == expected).all()

def test_upper_triangular():
    m = generate_upper_triangular(3)
    expected = [[1, 1, 1], [0, 1, 1], [0, 0, 1]]
    assert (m.data == expected).all()

def test_lower_triangular():
    m = generate_lower_triangular(3)
    expected = [[1, 0, 0], [1, 1, 0], [1, 1, 1]]
    assert (m.data == expected).all()

def test_tridiagonal():
    m = generate_tridiagonal(4, -1, 2, -1)
    expected = [[2, -1, 0, 0], [-1, 2, -1, 0], [0, -1, 2, -1], [0, 0, -1, 2]]
    assert (m.data == expected).all()

def test_triband():
    m = generate_triband(5, bandwidth=2, value=1)
    expected = [
        [1, 1, 1, 0, 0],
        [1, 1, 1, 1, 0],
        [1, 1, 1, 1, 1],
        [0, 1, 1, 1, 1],
        [0, 0, 1, 1, 1]
    ]
    assert (m.data == expected).all()

def test_diagonal():
    d = generate_diagonal([1, 2, 3])
    expected = [[1, 0, 0], [0, 2, 0], [0, 0, 3]]
    assert (d.data == expected).all()




from setuptools import setup, find_packages
from pathlib import Path

this_dir = Path(__file__).resolve().parent
long_description = (this_dir / "README.md").read_text(encoding="utf-8")

setup(
    name="matrixx",
    version="0.1.3",
    author="Kranthi",
    author_email="kdevprofile@gmail.com",
    description="Lightweight matrix utility library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/devk-op/matrix-utils",
    license="MIT",
    packages=find_packages(include=["matrixx", "matrixx.*"]),
    include_package_data=True,
    install_requires=["numpy"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
